@@ -22,7 +22,8 @@
 
 #include <QAction>
 
-#include <KDialog>
+#include <QDialog>
+#include <QPushButton>
 #include <KActionCollection>
 #include "cantor_macros.h"
 #include "backend.h"
@@ -34,11 +35,6 @@ CreateMatrixAssistant::CreateMatrixAssistant(QObject* parent, QList<QVariant> ar
     Q_UNUSED(args)
 }
 
-CreateMatrixAssistant::~CreateMatrixAssistant()
-{
-
-}
-
 void CreateMatrixAssistant::initActions()
 {
     setXMLFile(QLatin1String("cantor_create_matrix_assistant.rc"));
@@ -73,5 +69,5 @@ QStringList CreateMatrixAssistant::run(QWidget* parent)
     return result;
 }
 
-K_EXPORT_CANTOR_PLUGIN(creatematrixassistant, CreateMatrixAssistant)
+K_PLUGIN_FACTORY_WITH_JSON(creatematrixassistant, "creatematrixassistant.json", registerPlugin<CreateMatrixAssistant>();)
 #include "creatematrixassistant.moc"
