@@ -16,21 +16,17 @@
 
     ---
     Copyright (C) 2014 Lucas Hermann Negri <lucashnegri@gmail.com>
+    Copyright (C) 2019 Alexander Semke <alexander.semke@web.de>
  */
 
 #include "luabackend.h"
-#include "luasession.h"
 #include "luaextensions.h"
-#include "cantor_macros.h"
-
+#include "luasession.h"
 #include "settings.h"
 #include "ui_settings.h"
 
-#include <QFileInfo>
-
 LuaBackend::LuaBackend( QObject* parent,const QList<QVariant> args ) : Cantor::Backend( parent,args )
 {
-    setObjectName(QLatin1String("LuaBackend"));
     new LuaScriptExtension(this);
 }
 
@@ -58,21 +54,21 @@ Cantor::Backend::Capabilities LuaBackend::capabilities() const
     return cap;
 }
 
-bool LuaBackend::requirementsFullfilled() const
+bool LuaBackend::requirementsFullfilled(QString* const reason) const
 {
-    return QFileInfo(LuaSettings::self()->path().toLocalFile()).isExecutable();
+    const QString& path = LuaSettings::self()->path().toLocalFile();
+    return Cantor::Backend::checkExecutable(QLatin1String("Lua"), path, reason);
 }
 
 QUrl LuaBackend::helpUrl() const
 {
-    return QUrl(i18nc("Lua official documentation", "http://www.lua.org/docs.html"));
+    return QUrl(i18nc("Lua official documentation", "https://www.lua.org/docs.html"));
 }
 
 QString LuaBackend::description() const
 {
-    return i18n("<p> Lua is a fast and lightweight scripting language, with a simple procedural syntax." \
-                " There are several libraries in Lua aimed at math and science.</p>"
-                "<p>This backend supports luajit 2.</p>");
+    return i18n("<b>Lua</b> is a fast and lightweight scripting language, with a simple procedural syntax." \
+                " There are several libraries in Lua aimed at math and science.");
 }
 
 QWidget* LuaBackend::settingsWidget(QWidget* parent) const
