@@ -47,7 +47,7 @@ static QByteArray initCmd="os.environ['PAGER'] = 'cat'                     \n "\
                            "sage.misc.latex.EMBEDDED_MODE = True           \n "\
                            "os.environ['PAGER'] = 'cat'                    \n "\
                            "%colors nocolor                                \n "\
-                           "print '____TMP_DIR____', sage.misc.misc.SAGE_TMP\n";
+                           "print('%s %s' % ('____TMP_DIR____', sage.misc.misc.SAGE_TMP))\n";
 
 static QByteArray newInitCmd=
     "__CANTOR_IPYTHON_SHELL__=get_ipython()   \n "\
@@ -57,7 +57,7 @@ static QByteArray legacyInitCmd=
     "__CANTOR_IPYTHON_SHELL__=__IPYTHON__   \n "  \
     "__CANTOR_IPYTHON_SHELL__.autoindent=False\n ";
 
-static QByteArray endOfInitMarker="print '____END_OF_INIT____'\n ";
+static QByteArray endOfInitMarker="print('____END_OF_INIT____')\n ";
 
 
 
