@@ -17,26 +17,31 @@
     ---
     Copyright (C) 2009 Alexander Rieder <alexanderrieder@gmail.com>
     Copyright (C) 2012 Martin Kuettler <martin.kuettler@gmail.com>
-    Copyright (C) 2018 Alexander Semke <alexander.semke@web.de>
+    Copyright (C) 2018-2019 Alexander Semke <alexander.semke@web.de>
  */
 
 #include "commandentry.h"
 #include "resultitem.h"
 #include "loadedexpression.h"
+#include "lib/jupyterutils.h"
 #include "lib/result.h"
 #include "lib/helpresult.h"
+#include "lib/epsresult.h"
+#include "lib/latexresult.h"
 #include "lib/completionobject.h"
 #include "lib/syntaxhelpobject.h"
 #include "lib/session.h"
 
-#include <QApplication>
+#include <QGuiApplication>
 #include <QDebug>
 #include <QDesktopWidget>
 #include <QFontDialog>
+#include <QScreen>
 #include <QTimer>
 #include <QToolTip>
 #include <QPropertyAnimation>
-#include <QGraphicsWidget>
+#include <QJsonArray>
+#include <QJsonObject>
 
 #include <KLocalizedString>
 #include <KColorScheme>
@@ -76,7 +81,6 @@ CommandEntry::CommandEntry(Worksheet* worksheet) : WorksheetEntry(worksheet),
     m_textColorMenu(nullptr),
     m_fontMenu(nullptr)
 {
-
     m_promptItem->setPlainText(Prompt);
     m_promptItem->setItemDragable(true);
     m_commandItem->enableCompletion(true);
@@ -84,7 +88,7 @@ CommandEntry::CommandEntry(Worksheet* worksheet) : WorksheetEntry(worksheet),
     KColorScheme scheme = KColorScheme(QPalette::Normal, KColorScheme::View);
     m_commandItem->setBackgroundColor(scheme.background(KColorScheme::AlternateBackground).color());
 
-    m_promptItemAnimation = new QPropertyAnimation(m_promptItem, "opacity");
+    m_promptItemAnimation = new QPropertyAnimation(m_promptItem, "opacity", this);
     m_promptItemAnimation->setDuration(600);
     m_promptItemAnimation->setStartValue(1);
     m_promptItemAnimation->setKeyValueAt(0.5, 0);
@@ -94,12 +98,12 @@ CommandEntry::CommandEntry(Worksheet* worksheet) : WorksheetEntry(worksheet),
     connect(m_commandItem, &WorksheetTextItem::tabPressed, this, &CommandEntry::showCompletion);
     connect(m_commandItem, &WorksheetTextItem::backtabPressed, this, &CommandEntry::selectPreviousCompletion);
     connect(m_commandItem, &WorksheetTextItem::applyCompletion, this, &CommandEntry::applySelectedCompletion);
-    connect(m_commandItem, SIGNAL(execute()), this, SLOT(evaluate()));
+    connect(m_commandItem, &WorksheetTextItem::execute, this, [=]() { evaluate();} );
     connect(m_commandItem, &WorksheetTextItem::moveToPrevious, this, &CommandEntry::moveToPreviousItem);
     connect(m_commandItem, &WorksheetTextItem::moveToNext, this, &CommandEntry::moveToNextItem);
-    connect(m_commandItem, SIGNAL(receivedFocus(WorksheetTextItem*)), worksheet, SLOT(highlightItem(WorksheetTextItem*)));
+    connect(m_commandItem, &WorksheetTextItem::receivedFocus, worksheet, &Worksheet::highlightItem);
     connect(m_promptItem, &WorksheetTextItem::drag, this, &CommandEntry::startDrag);
-    connect(worksheet, SIGNAL(updatePrompt()), this, SLOT(updatePrompt()));
+    connect(worksheet, &Worksheet::updatePrompt, this, [=]() { updatePrompt();} );
 }
 
 CommandEntry::~CommandEntry()
@@ -197,17 +201,17 @@ void CommandEntry::initMenus() {
 }
 
 void CommandEntry::backgroundColorChanged(QAction* action) {
-	int index = m_backgroundColorActionGroup->actions().indexOf(action);
-	if (index == -1 || index>=colorsCount)
-		index = 0;
+    int index = m_backgroundColorActionGroup->actions().indexOf(action);
+    if (index == -1 || index>=colorsCount)
+        index = 0;
 
     m_commandItem->setBackgroundColor(colors[index]);
 }
 
 void CommandEntry::textColorChanged(QAction* action) {
-	int index = m_textColorActionGroup->actions().indexOf(action);
-	if (index == -1 || index>=colorsCount)
-		index = 0;
+    int index = m_textColorActionGroup->actions().indexOf(action);
+    if (index == -1 || index>=colorsCount)
+        index = 0;
 
     m_commandItem->setDefaultTextColor(colors[index]);
 }
@@ -360,7 +364,7 @@ void CommandEntry::setExpression(Cantor::Expression* expr)
         m_errorItem = nullptr;
     }
 
-    foreach(WorksheetTextItem* item, m_informationItems)
+    for (auto* item : m_informationItems)
     {
         item->deleteLater();
     }
@@ -372,14 +376,14 @@ void CommandEntry::setExpression(Cantor::Expression* expr)
     m_expression = expr;
     m_resultsCollapsed = false;
 
-    connect(expr, SIGNAL(gotResult()), this, SLOT(updateEntry()));
-    connect(expr, SIGNAL(resultsCleared()), this, SLOT(clearResultItems()));
-    connect(expr, SIGNAL(resultRemoved(int)), this, SLOT(removeResultItem(int)));
-    connect(expr, SIGNAL(resultReplaced(int)), this, SLOT(replaceResultItem(int)));
-    connect(expr, SIGNAL(idChanged()), this, SLOT(updatePrompt()));
-    connect(expr, SIGNAL(statusChanged(Cantor::Expression::Status)), this, SLOT(expressionChangedStatus(Cantor::Expression::Status)));
-    connect(expr, SIGNAL(needsAdditionalInformation(QString)), this, SLOT(showAdditionalInformationPrompt(QString)));
-    connect(expr, SIGNAL(statusChanged(Cantor::Expression::Status)), this, SLOT(updatePrompt()));
+    connect(expr, &Cantor::Expression::gotResult, this, &CommandEntry::updateEntry);
+    connect(expr, &Cantor::Expression::resultsCleared, this, &CommandEntry::clearResultItems);
+    connect(expr, &Cantor::Expression::resultRemoved, this, &CommandEntry::removeResultItem);
+    connect(expr, &Cantor::Expression::resultReplaced, this, &CommandEntry::replaceResultItem);
+    connect(expr, &Cantor::Expression::idChanged, this,  [=]() { updatePrompt();} );
+    connect(expr, &Cantor::Expression::statusChanged, this, &CommandEntry::expressionChangedStatus);
+    connect(expr, &Cantor::Expression::needsAdditionalInformation, this, &CommandEntry::showAdditionalInformationPrompt);
+    connect(expr, &Cantor::Expression::statusChanged, this,  [=]() { updatePrompt();} );
 
     updatePrompt();
 
@@ -411,7 +415,7 @@ void CommandEntry::setContent(const QDomElement& content, const KZip& file)
 {
     m_commandItem->setPlainText(content.firstChildElement(QLatin1String("Command")).text());
 
-    LoadedExpression* expr=new LoadedExpression( worksheet()->session() );
+    LoadedExpression* expr = new LoadedExpression( worksheet()->session() );
     expr->loadFromXml(content, file);
 
     //background color
@@ -450,6 +454,94 @@ void CommandEntry::setContent(const QDomElement& content, const KZip& file)
     setExpression(expr);
 }
 
+void CommandEntry::setContentFromJupyter(const QJsonObject& cell)
+{
+    m_commandItem->setPlainText(Cantor::JupyterUtils::getSource(cell));
+
+    LoadedExpression* expr=new LoadedExpression( worksheet()->session() );
+    expr->loadFromJupyter(cell);
+    setExpression(expr);
+
+    // https://nbformat.readthedocs.io/en/latest/format_description.html#cell-metadata
+    // 'collapsed': +
+    // 'scrolled', 'deletable', 'name', 'tags' don't supported Cantor, so ignore them
+    // 'source_hidden' don't supported
+    // 'format' for raw entry, so ignore
+    // I haven't found 'outputs_hidden' inside Jupyter notebooks, and difference from 'collapsed'
+    // not clear, so also ignore
+    const QJsonObject& metadata = Cantor::JupyterUtils::getMetadata(cell);
+    const QJsonValue& collapsed = metadata.value(QLatin1String("collapsed"));
+    if (collapsed.isBool() && collapsed.toBool() == true && !m_resultItems.isEmpty())
+    {
+        // Disable animation for hiding results, we don't need animation on document load stage
+        bool animationValue = worksheet()->animationsEnabled();
+        worksheet()->enableAnimations(false);
+        collapseResults();
+        worksheet()->enableAnimations(animationValue);
+    }
+
+    setJupyterMetadata(metadata);
+}
+
+QJsonValue CommandEntry::toJupyterJson()
+{
+    QJsonObject entry;
+
+    entry.insert(QLatin1String("cell_type"), QLatin1String("code"));
+
+    QJsonValue executionCountValue;
+    if (expression() && expression()->id() != -1)
+        executionCountValue = QJsonValue(expression()->id());
+    entry.insert(QLatin1String("execution_count"), executionCountValue);
+
+    QJsonObject metadata(jupyterMetadata());
+    if (m_resultsCollapsed)
+        metadata.insert(QLatin1String("collapsed"), true);
+
+    entry.insert(QLatin1String("metadata"), metadata);
+
+    Cantor::JupyterUtils::setSource(entry, command());
+
+    QJsonArray outputs;
+    if (expression())
+    {
+        Cantor::Expression::Status status = expression()->status();
+        if (status == Cantor::Expression::Error || status == Cantor::Expression::Interrupted)
+        {
+            QJsonObject errorOutput;
+            errorOutput.insert(Cantor::JupyterUtils::outputTypeKey, QLatin1String("error"));
+            errorOutput.insert(QLatin1String("ename"), QLatin1String("Unknown"));
+            errorOutput.insert(QLatin1String("evalue"), QLatin1String("Unknown"));
+
+            QJsonArray traceback;
+            if (status == Cantor::Expression::Error)
+            {
+                const QStringList& error = expression()->errorMessage().split(QLatin1Char('\n'));
+                for (const QString& line: error)
+                    traceback.append(line);
+            }
+            else
+            {
+                traceback.append(i18n("Interrupted"));
+            }
+            errorOutput.insert(QLatin1String("traceback"), traceback);
+
+            outputs.append(errorOutput);
+        }
+
+        for (auto* result : expression()->results())
+        {
+            const QJsonValue& resultJson = result->toJupyterJson();
+
+            if (!resultJson.isNull())
+                outputs.append(resultJson);
+        }
+    }
+    entry.insert(QLatin1String("outputs"), outputs);
+
+    return entry;
+}
+
 void CommandEntry::showCompletion()
 {
     const QString line = currentLine();
@@ -515,7 +607,7 @@ QDomElement CommandEntry::toXml(QDomDocument& doc, KZip* archive)
             errorElem.appendChild(doc.createTextNode(errorMessage));
             exprElem.appendChild(errorElem);
         }
-        for (Cantor::Result * const result: expression()->results())
+        for (auto* result : expression()->results())
         {
             const QDomElement& resultElem = result->toXml(doc);
             exprElem.appendChild(resultElem);
@@ -602,7 +694,7 @@ bool CommandEntry::evaluate(EvaluationOption evalOp)
 
     if(cmd.isEmpty()) {
         removeResults();
-        foreach(WorksheetTextItem* item, m_informationItems) {
+        for (auto* item : m_informationItems) {
             item->deleteLater();
         }
         m_informationItems.clear();
@@ -612,9 +704,8 @@ bool CommandEntry::evaluate(EvaluationOption evalOp)
         return false;
     }
 
-    Cantor::Expression* expr;
-    expr = worksheet()->session()->evaluateExpression(cmd);
-    connect(expr, SIGNAL(gotResult()), worksheet(), SLOT(gotResult()));
+    Cantor::Expression* expr = worksheet()->session()->evaluateExpression(cmd);
+    connect(expr, &Cantor::Expression::gotResult, this, [=]() { worksheet()->gotResult(expr); });
 
     setExpression(expr);
 
@@ -779,8 +870,9 @@ void CommandEntry::showCompletions()
             m_completionBox->setCurrentItem(items.first());
         m_completionBox->setTabHandling(false);
         m_completionBox->setActivateOnSelect(true);
+
         connect(m_completionBox.data(), &KCompletionBox::activated, this, &CommandEntry::applySelectedCompletion);
-        connect(m_commandItem->document(), SIGNAL(contentsChanged()), this, SLOT(completedLineChanged()));
+        connect(m_commandItem->document(), &QTextDocument::contentsChanged, this, &CommandEntry::completedLineChanged);
         connect(m_completionObject, &Cantor::CompletionObject::done, this, &CommandEntry::updateCompletions);
 
         m_commandItem->activateCompletion(true);
@@ -898,9 +990,9 @@ void CommandEntry::showSyntaxHelp()
     QString msg = m_syntaxHelpObject->toHtml();
     const QPointF cursorPos = m_commandItem->cursorPosition();
 
-    // QToolTip don't support &nbsp;, but support multiple spaces
+    // QToolTip doesn't support &nbsp;, but support multiple spaces
     msg.replace(QLatin1String("&nbsp;"), QLatin1String(" "));
-    // Don't support &quot too;
+    // Doesn't support &quot, neither;
     msg.replace(QLatin1String("&quot;"), QLatin1String("\""));
 
     QToolTip::showText(toGlobalPosition(cursorPos), msg, worksheetView());
@@ -1005,9 +1097,9 @@ void CommandEntry::removeContextHelp()
 
 void CommandEntry::updatePrompt(const QString& postfix)
 {
-    KColorScheme color = KColorScheme( QPalette::Normal, KColorScheme::View);
+    KColorScheme color = KColorScheme(QPalette::Normal, KColorScheme::View);
 
-    m_promptItem->setPlainText(QLatin1String(""));
+    m_promptItem->setPlainText(QString());
     QTextCursor c = m_promptItem->textCursor();
     QTextCharFormat cformat = c.charFormat();
 
@@ -1062,8 +1154,8 @@ QPoint CommandEntry::getPopupPosition()
 {
     const QPointF cursorPos = m_commandItem->cursorPosition();
     const QPoint globalPos = toGlobalPosition(cursorPos);
-    const QDesktopWidget* desktop = QApplication::desktop();
-    const QRect screenRect = desktop->screenGeometry(globalPos);
+    const QScreen* desktop = QGuiApplication::primaryScreen();
+    const QRect screenRect = desktop->geometry();
     if (globalPos.y() + m_completionBox->height() < screenRect.bottom()) {
         return (globalPos);
     } else {
@@ -1238,5 +1330,3 @@ void CommandEntry::setMidPrompt()
 {
     updatePrompt(MidPrompt);
 }
-
-
