@@ -24,21 +24,27 @@ using namespace Cantor;
 #include <config-cantorlib.h>
 
 #include <QDebug>
+#include <QJsonValue>
+
 #include <KZip>
 #include <KIO/Job>
+#include <QBuffer>
+#include <QJsonObject>
+
+#include "renderer.h"
+#include "jupyterutils.h"
 
 class Cantor::EpsResultPrivate{
     public:
         QUrl url;
+        QImage image;
 };
 
 
-EpsResult::EpsResult(const QUrl& url) : d(new EpsResultPrivate)
+EpsResult::EpsResult(const QUrl& url, const QImage& image) : d(new EpsResultPrivate)
 {
     d->url=url;
-#ifndef WITH_EPS
-    qDebug()<<"Creating an EpsResult in an environment compiled without EPS support!";
-#endif
+    d->image = image;
 }
 
 EpsResult::~EpsResult()
@@ -66,6 +72,11 @@ QUrl EpsResult::url()
     return d->url;
 }
 
+QImage Cantor::EpsResult::image()
+{
+    return d->image;
+}
+
 int EpsResult::type()
 {
     return EpsResult::Type;
@@ -80,13 +91,56 @@ QDomElement EpsResult::toXml(QDomDocument& doc)
 {
     qDebug()<<"saving imageresult "<<toHtml();
     QDomElement e=doc.createElement(QStringLiteral("Result"));
-    e.setAttribute(QStringLiteral("type"), QStringLiteral("image"));
+    e.setAttribute(QStringLiteral("type"), QStringLiteral("epsimage"));
     e.setAttribute(QStringLiteral("filename"), d->url.fileName());
-    qDebug()<<"done";
 
+#ifdef WITH_EPS
+    const QImage& image = Renderer::epsRenderToImage(d->url, 1.0, false);
+    qDebug() << image.size() << image.isNull();
+    if (!image.isNull())
+    {
+        QByteArray ba;
+        QBuffer buffer(&ba);
+        buffer.open(QIODevice::WriteOnly);
+        image.save(&buffer, "PNG");
+        e.setAttribute(QLatin1String("image"), QString::fromLatin1(ba.toBase64()));
+    }
+#else
+    if (!d->image.isNull())
+    {
+        QByteArray ba;
+        QBuffer buffer(&ba);
+        buffer.open(QIODevice::WriteOnly);
+        d->image.save(&buffer, "PNG");
+        e.setAttribute(QLatin1String("image"), QString::fromLatin1(ba.toBase64()));
+    }
+#endif
+
+    qDebug()<<"done";
     return e;
 }
 
+QJsonValue Cantor::EpsResult::toJupyterJson()
+{
+    QJsonObject root;
+
+    if (executionIndex() != -1)
+    {
+        root.insert(QLatin1String("output_type"), QLatin1String("execute_result"));
+        root.insert(QLatin1String("execution_count"), executionIndex());
+    }
+    else
+        root.insert(QLatin1String("output_type"), QLatin1String("display_data"));
+
+    const QImage& image = d->image.isNull() ? Renderer::epsRenderToImage(d->url, 1.0, false) : d->image;
+
+    QJsonObject data;
+    data.insert(JupyterUtils::pngMime, JupyterUtils::packMimeBundle(image, JupyterUtils::pngMime));
+    root.insert(QLatin1String("data"), data);
+
+    return root;
+}
+
 void EpsResult::saveAdditionalData(KZip* archive)
 {
     archive->addLocalFile(d->url.toLocalFile(), d->url.fileName());
